
# q-expansion of newform 5390.2.a.bf, downloaded from the LMFDB on 15 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5390
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1], [1], [-1], [0], [1], [6], [7], [-5], [-6], [5], [3], [3], [-2], [4], [2], [-1], [10], [-7], [8], [7], [-14], [10], [6], [15], [12], [-2], [-4], [8], [-10], [-16], [8], [-17], [-12], [20], [15], [2], [-3], [19], [-3], [-14], [0], [-2], [12], [-11], [-12], [25], [-23], [6], [2], [-10], [9], [10], [18], [-2], [2], [9], [20], [8], [-12], [-18], [6], [6], [2], [3], [6], [-7], [-28], [-17], [18], [-30], [-34], [-20], [-28], [-6], [-30], [-34], [-30], [2], [-13], [20], [0], [32], [-8], [16], [-20], [4], [-30], [3], [-27], [34], [-23], [0], [-12], [-3], [20], [-24], [-20], [-22], [16], [-23], [8], [18], [6], [0], [27], [-38], [27], [-14], [45], [-42], [47], [-26], [8], [-20], [-33], [-33], [-19], [42], [-31], [15], [-2], [29], [-28], [-31], [38], [7], [-10], [-25], [22], [-24], [40], [-21], [17], [38], [18], [10], [-19], [-28], [-18], [-30], [-7], [-18], [-16], [8], [20], [0], [26], [7], [-30], [-6], [38], [-2], [9], [-8], [-57], [27], [40], [-35], [-38], [-17], [-27], [39], [-27], [48], [-12], [-54], [-8], [32], [50], [21], [-35], [18], [-18], [-11], [0], [5], [-28], [12], [-9], [45], [38], [-57], [29], [27], [34], [-10], [-52], [-4], [-50], [48], [-24], [4], [-38], [28], [-12], [6], [32], [34], [22], [-49], [5], [18], [27], [40], [-20], [-43], [40], [24], [-30], [-32], [-37], [-52], [4], [27], [-40], [13], [33], [18], [48], [-51], [-18], [-10], [-35], [4], [32], [55], [46], [35], [-8], [52], [-46], [-55], [22], [-18], [-4], [47], [5], [54], [-20], [-57], [29], [-12], [-4], [70], [6], [10], [52], [-42], [-55], [-71], [38], [-52], [-2], [5], [-24], [25], [-28], [-18], [2], [17], [-16], [-52], [-35], [26], [12], [-80], [80], [-57], [-16], [-6], [-22], [33], [56], [-50], [7], [-54], [-32], [10], [22], [-27], [61], [52], [32], [33], [-68], [-43], [-26], [-2], [5], [30], [52], [12], [-11], [-7], [34], [30], [48], [-69], [-60], [2], [21], [28], [-20], [-36], [-38], [88], [68], [40], [-15], [-56], [-44], [-70], [52], [19], [43], [50], [10], [-8], [66], [20], [73], [38], [38], [-46], [39], [13], [-10], [36], [63], [24], [17], [-22], [40], [-34], [37], [57], [90], [31], [43], [-23], [34], [58], [55], [87], [39], [35], [-52], [53], [28], [87], [23], [38], [-68], [31], [55], [-39], [-65], [-47], [-82], [44], [-2], [-72], [28], [-25], [-3], [-56], [-18], [-1], [-8], [32], [20], [-96], [-65], [-62], [78], [0], [42], [-49], [0], [-43], [48], [-26], [-92], [-97], [0], [-64], [-88], [67], [-76], [82], [5], [-9], [0], [-8], [-8], [74], [-10], [-15], [2], [87], [95], [-6], [28], [-48], [-30], [-42], [-7], [-58], [-9], [-90], [86], [-32], [84], [52], [38], [8], [-60], [-8], [-38], [-39], [-60], [32], [-52], [-45], [26], [87], [29], [80], [-52], [10]]
